#include "image.h"
#include <glm/gtc/noise.hpp>
#include "lodepng.h"

Image::Image(int myDepth, int myWidth, int myHeight, float frequency, float amplitude) :
    width(myWidth),
    height(myHeight),
    depth(myDepth)
{
    allocate(width * height * (depth/8));
    unsigned char* data = pixels.data();

    float xFactor = 1.0f / (width - 1);
    float yFactor = 1.0f / (height - 1);

    for( int row = 0; row < height; row++ ) {
        for( int col = 0 ; col < width; col++ ) {
            float x = xFactor * col;
            float y = yFactor * row;
            float sum = 0.0f;
            float freq = frequency;
            float scale = amplitude;

            // Compute the sum for each octave
            for( int oct = 0; oct < 4 ; oct++ ) {
                glm::vec2 p(x * freq, y * freq);
                float val = glm::perlin(p, glm::vec2(freq)) / scale;
                sum += val;
                float result = (sum + 1.0f)/ 2.0f;

                // Store in texture buffer
                data[((row * width + col) * (depth/8)) + oct] =
                      (unsigned char) ( result * 255.0f );
                freq *= 2.0f;   // Double the frequency
                scale *= amplitude;     // Next power of b
            }
        }
    }
}

Image::Image(const std::string &pngFile)
{
    unsigned int w, h;
    unsigned error = lodepng::decode(pixels, w, h, pngFile);
    if(error)
        fprintf(stderr, lodepng_error_text(error));
    else
    {
        width = w;
        height = h;
        depth = pixels.size()*8/(width*height);
    }
}

bool Image::save(const std::string &filename)
{
    LodePNGColorType colorType;
    switch(depth)
    {
        case 32: colorType = LCT_RGBA; break;
        case 24: colorType = LCT_RGB; break;
        case 8: colorType = LCT_GREY; break;
        default: return false; break;
    }
    unsigned error = lodepng::encode(filename, pixels.data(), width, height, colorType, 8);
    if(error)
        fprintf(stderr, lodepng_error_text(error));
    return error != 0;
}
