#include "phongmaterial.h"
#include "texture.h"
#include "sparrowrenderer.h"

#include "shader.h"
#include "mesh.h"
#include <glm/ext.hpp>

void PhongMaterial::bindAttributes(Shader* myShader)
{
    // TODO store the attributes location
    myShader->bindFloat(myShader->getLocation("materialNs"), shininess);

    if(textures[NORMALS_SLOT] != NULL)
    {
        textures[NORMALS_SLOT]->bind(NORMALS_SLOT);
        myShader->bindInteger(myShader->getLocation("normalMap"), NORMALS_SLOT);
    }

    if(textures[EMISSION_SLOT] != NULL)
    {
        textures[EMISSION_SLOT]->bind(EMISSION_SLOT);
        myShader->bindInteger(myShader->getLocation("emissionTexture"), EMISSION_SLOT);
    }
    else
        myShader->bindVec3(myShader->getLocation("materialEmission"), emission);

    if(textures[DIFFUSE_SLOT] != NULL)
    {
        textures[DIFFUSE_SLOT]->bind(DIFFUSE_SLOT);
        myShader->bindInteger(myShader->getLocation("diffuseTexture"), DIFFUSE_SLOT);
    }
    else
        myShader->bindVec3(myShader->getLocation("materialKd"), diffuse);

    if(textures[SPECULAR_SLOT] != NULL)
    {
        textures[SPECULAR_SLOT]->bind(SPECULAR_SLOT);
        myShader->bindInteger(myShader->getLocation("specularTexture"), SPECULAR_SLOT);
    }
    else
        myShader->bindVec3(myShader->getLocation("materialKs"), specular);

    if(textures[ALPHA_SLOT] != NULL)
    {
        textures[ALPHA_SLOT]->bind(ALPHA_SLOT);
        myShader->bindInteger(myShader->getLocation("alphaMask"), ALPHA_SLOT);
    }
}

unsigned int PhongMaterial::getFlags()
{
    unsigned int flags = 1 << Mesh::MATERIAL_PHONG;
    if(textures[NORMALS_SLOT] != NULL)
        flags |= 1 << Mesh::MATERIAL_PHONG_NORMAL_MAP;
    if(textures[EMISSION_SLOT] != NULL)
        flags |= 1 << Mesh::MATERIAL_PHONG_EMISSION_TEXTURE;
    if(textures[DIFFUSE_SLOT] != NULL)
        flags |= 1 << Mesh::MATERIAL_PHONG_DIFFUSE_TEXTURE;
    if(textures[SPECULAR_SLOT] != NULL)
        flags |= 1 << Mesh::MATERIAL_PHONG_SPECULAR_TEXTURE;
    if(textures[ALPHA_SLOT] != NULL)
        flags |= 1 << Mesh::MATERIAL_ALPHA_MASK;
    return flags;
}

